package org.openscm.kundo.plugins

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.plugins.context.BuildContext
 
/**
 * Clean Delegate
 * @author <a href="mailto:steve.king@upco.co.uk">Steve king</a>
 * @version 1.0.0
 * 
 * Description: Clean Delegate removes output directories that have been loaded from 
 * either the plugin.properties or the build.properties
 * 
 * <p>The delegate performs a two phases clean:
 * 1 - All of the output directories from the global indexed project properties are removed
 * 2 - A generic list of directories from the comma delimited clean.paths property are removed</p>
 */
/* 
 * @ant-target doClean
 */
class CleanDelegate extends AbstractPluginTargetDelegate {

	/*
	 * a comma separated list of paths to clean additional to previously global project paths
	 * @ant-property-name clean.paths
	 */	
	String paths
	
	/**
	 * Constructor sets ant and buildContext instances in super class
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	CleanDelegate( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
	}
	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */	
	void doCheck(){
		assert paths != null
		assert buildContext.get( "project.properties" ) != null
	}	
	
	/**
	* Injector for Paths element
	*/
	void setPaths( String paths ){
		this.paths = paths
	}	
	
	/**
	 * Private method for deleting generic paths
	 * @param paths - List of string paths to be removed
	 */
	def clean(paths) {
		 
		 // For each path in the list, clean the output dir
		 paths.each{ path -> 
		 	if ( !path.equals( "" ) ){
			 	 println "Clean directory: $path"
			 	 ant.delete( dir:path )
		 	}
		 }
	}

	/**
	 * Private method for deleting indexed paths retrieved from the project properties
	 * @param paths - List of dynamic objects containing output paths to be deleted
	 */
	def cleanIndexedPaths(paths) {
		// For each Expando in the list
		paths.each{ path ->
			// Check to see if there is a specified output directory
			if ( path.output != null )	{
				// TODO Swop to logger
				// delete the directory
				println "Deleting directory: $path.output"
				ant.delete( dir:path.output )
			}
		}
	}
	
	/**
	 * Concrete implementation of the AbstractDelegate.execute() method
	 */
	void doExecute(){		
		
		// Step 1 - Clean Indexed Paths
		// Retrieve the indexed project properties from the global context
		Map properties = buildContext.get("project.properties")
		// For each keyed path set
		properties.each { key, value ->
			// clean the paths
			cleanIndexedPaths( value )
		}
		
		
		// Step 2 - Clean Generic Paths
		// Take the injected set of paths. Must be comma delimited.. 
		
		println "paths: $paths"
		if( paths != null )  {
			// Split the generic project paths and process
			clean( paths.split( ',' ) )
		}
	}
}